@@ -43,7 +43,7 @@ static void wfs_gml_bounded_by(ows * o, wfs_request * wr, double xmin, double ym
   assert(wr);
   assert(srs);
 
-  if (srs->is_degree) precision = o->degree_precision;
+  if (srs->is_geographic) precision = o->degree_precision;
   else precision = o->meter_precision;
 
   if (!srs->srid || srs->srid == -1 || (xmin == ymin && xmax == ymax && xmin == xmax)) {
@@ -76,7 +76,7 @@ static void wfs_gml_bounded_by(ows * o, wfs_request * wr, double xmin, double ym
       else fprintf(o->output, "EPSG:");
       fprintf(o->output, "%d\">", srs->srid);
 
-      if (srs->is_reverse_axis && !srs->is_eastern_axis && srs->is_long) {
+      if (srs->honours_authority_axis_order && !srs->is_axis_order_gis_friendly) {
         if (fabs(xmin) > OWS_MAX_DOUBLE || fabs(ymin) > OWS_MAX_DOUBLE ||
             fabs(xmax) > OWS_MAX_DOUBLE || fabs(ymax) > OWS_MAX_DOUBLE) {
           fprintf(o->output, "<gml:lowerCorner>%g %g</gml:lowerCorner>", ymin, xmin);
@@ -237,7 +237,6 @@ static void wfs_gml_display_namespaces(ows * o, wfs_request * wr)
   array *namespaces;
   array_node *an;
   list_node *ln;
-  buffer * ns_prefix;
 
   assert(o);
   assert(wr);
@@ -276,8 +275,7 @@ static void wfs_gml_display_namespaces(ows * o, wfs_request * wr)
   if (wr->typename) {
     fprintf(o->output, "&amp;Typename=");
     for (ln = wr->typename->first ; ln ; ln = ln->next) {
-      ns_prefix = ows_layer_ns_prefix(o->layers, ln->value);
-      fprintf(o->output, "%s:%s", ns_prefix->buf, ln->value->buf);
+      fprintf(o->output, "%s", ln->value->buf);
       if (ln->next) fprintf(o->output, ",");
     }
   }
@@ -459,15 +457,18 @@ static buffer *wfs_retrieve_sql_request_select(ows * o, wfs_request * wr, buffer
           buffer_add_str(select, "\",");
         }
 
-        if ((wr->srs && !wr->srs->is_degree) ||
+        if ((wr->srs && !wr->srs->is_geographic) ||
             (!wr->srs && ows_srs_meter_units(o, layer_name)))
           buffer_add_int(select, o->meter_precision);
         else
           buffer_add_int(select, o->degree_precision);
 
                                                  gml_opt  = 0; /* Short SRS */
-        if (wr->srs && wr->srs->is_long)         gml_opt += 1; /* FIXME really ? */
-        if (wr->srs && wr->srs->is_eastern_axis) gml_opt += 16;
+        if (wr->srs && wr->srs->is_long)         gml_opt += 1;
+        /* This will be actually without effect since PostGIS only honours flag = 16 for GML 3 */
+        if (wr->srs &&
+            wr->srs->honours_authority_axis_order &&
+            !wr->srs->is_axis_order_gis_friendly ) gml_opt += 16;
         if (gml_boundedby)                       gml_opt += 32;
 
         buffer_add_str(select, ",");
@@ -498,20 +499,21 @@ static buffer *wfs_retrieve_sql_request_select(ows * o, wfs_request * wr, buffer
         if (wr->srs && wr->srs->is_long) gml_opt += 1; /* Long SRS */
         if (gml_boundedby) gml_opt += 32;
 
-        if ((wr->srs && !wr->srs->is_degree) || (!wr->srs && ows_srs_meter_units(o, layer_name))) {
+        if ((wr->srs && !wr->srs->is_geographic) || (!wr->srs && ows_srs_meter_units(o, layer_name))) {
           buffer_add_int(select, o->meter_precision);
-          if (wr->srs && !wr->srs->is_eastern_axis && wr->srs->is_long) gml_opt += 16;
         } else {
           buffer_add_int(select, o->degree_precision);
-          if (wr->srs && !wr->srs->is_eastern_axis && wr->srs->is_long) gml_opt += 16;
         }
+        if (wr->srs &&
+            wr->srs->honours_authority_axis_order &&
+            !wr->srs->is_axis_order_gis_friendly ) gml_opt += 16;
 
         buffer_add_str(select, ", ");
         buffer_add_int(select, gml_opt);
         buffer_add_str(select, ") AS \"");
         buffer_copy(select, an->key);
         buffer_add_str(select, "\" ");
-      } else if (wr->format == WFS_GEOJSON) {
+      } else if (wr->format == WFS_GEOJSON || wr->format == WFS_JSONP) {
         buffer_add_str(select, "ST_AsGeoJSON(");
 
         /* Geometry Reprojection on the fly step if asked */
@@ -528,7 +530,7 @@ static buffer *wfs_retrieve_sql_request_select(ows * o, wfs_request * wr, buffer
           buffer_add_str(select, "\",");
         }
 
-        if ((wr->srs && !wr->srs->is_degree) ||
+        if ((wr->srs && !wr->srs->is_geographic) ||
             (!wr->srs && ows_srs_meter_units(o, layer_name)))
           buffer_add_int(select, o->meter_precision);
         else
@@ -543,9 +545,13 @@ static buffer *wfs_retrieve_sql_request_select(ows * o, wfs_request * wr, buffer
     }
     /* Columns are written in quotation marks */
     else {
+      if (wr->format == WFS_GEOJSON)
+          buffer_add_str(select, "to_json(");
       buffer_add_str(select, "\"");
       buffer_copy(select, an->key);
       buffer_add_str(select, "\"");
+      if (wr->format == WFS_GEOJSON)
+          buffer_add_str(select, ")");
     }
 
     if (an->next) buffer_add_str(select, ",");
@@ -740,6 +746,8 @@ static mlist *wfs_retrieve_sql_request_list(ows * o, wfs_request * wr)
 
     buffer_copy(sql, where);
 
+    /* fprintf(stderr, "sql = %s\n", sql->buf); */
+
     list_add(sql_req, sql);
     list_add(where_list, where);
     list_add_by_copy(from_list, layer_uri);
@@ -786,7 +794,17 @@ static void wfs_geojson_display_results(ows * o, wfs_request * wr, mlist * reque
   prop = buffer_init();
   id_name = buffer_init();
 
-  fprintf(o->output, "Content-Type: application/json\n\n");
+  if (wr->format == WFS_JSONP)
+  {
+         assert(wr->callback);
+
+         fprintf(o->output, "Content-Type: application/javascript\n\n");
+         fprintf(o->output, "%s", wr->callback->buf);
+         fprintf(o->output, "(");
+
+  } else fprintf(o->output, "Content-Type: application/json\n\n");
+  
+
   fprintf(o->output, "{\"type\": \"FeatureCollection\", \"crs\":{\"type\":\"name\",\"properties\":{\"name\":\"");
   if (ows_version_get(o->request->version) == 100)
     fprintf(o->output, "EPSG:%i", wr->srs->srid);
@@ -806,7 +824,8 @@ static void wfs_geojson_display_results(ows * o, wfs_request * wr, mlist * reque
 
     prop_table = ows_psql_describe_table(o, ll->value);
     first_row = true;
-    id_name = ows_psql_id_column(o, ll->value);
+    if(ows_psql_id_column(o, ll->value)) /* CAUTION: pkey could be NULL ! */
+      buffer_copy(id_name, ows_psql_id_column(o, ll->value));
     number = -1;
     if (id_name && id_name->use)
          number = PQfnumber(res, id_name->buf);
@@ -822,7 +841,7 @@ static void wfs_geojson_display_results(ows * o, wfs_request * wr, mlist * reque
 
       if ( number >= 0 ) {
         buffer_add_str(id_name, "\"id\": \"");
-        buffer_copy(id_name, ll->value);
+        buffer_copy(id_name, ows_layer_no_uri(o->layers, ll->value));
         buffer_add_str(id_name, ".");
         buffer_add_str(id_name, PQgetvalue(res, i, number));
         buffer_add_str(id_name, "\", ");
@@ -838,11 +857,10 @@ static void wfs_geojson_display_results(ows * o, wfs_request * wr, mlist * reque
           else buffer_add_str(prop, ", \"");
 
           buffer_copy(prop, an->key);
-          buffer_add_str(prop, "\": \"");
+          buffer_add_str(prop, "\": ");
           value_enc = buffer_encode_json_str(PQgetvalue(res, i, j));
           buffer_copy(prop, value_enc);
           buffer_free(value_enc);
-          buffer_add(prop, '"');
         }
       }
 
@@ -871,6 +889,7 @@ static void wfs_geojson_display_results(ows * o, wfs_request * wr, mlist * reque
   }
 
   fprintf(o->output, "]}");
+  if (wr->format == WFS_JSONP) fprintf(o->output, ");");
 
   buffer_free(geom);
   buffer_free(prop);
@@ -896,7 +915,7 @@ void wfs_get_feature(ows * o, wfs_request * wr)
     else
       wfs_gml_display_results(o, wr, request_list);
 
-  } else if (wr->format == WFS_GEOJSON)
+  } else if (wr->format == WFS_GEOJSON || wr->format == WFS_JSONP)
     wfs_geojson_display_results(o, wr, request_list);
 
   /* Add here other functions to display GetFeature response in other formats */
